/*
Firmware for 1wireTooltageConverter in C

Used CPU: PIC12F1572
Internal clock with 16Mhz

Converts Values from 1 wire device like the DS18B20
to analoque voltages

Copyright by Ramser Elektrotechnik 2022

Revision: 0

R0 @FR221216:
Rewrite in mikroC Pro, because "mikroe mikrobasic pro" is to bad for the mikrocontroller.net forum

R0 @FR221202:
Forum Edition - First Realease
-20C - +80C = 0V - 10V


Hardware assignment:
RA0 = PWM2 + TX
RA1 = RX
RA2 = PWM3
RA3 = 7.5V Adjust Jumper
RA4 = 1Wire1
RA5 = 1Wire2
*/

#include<built_in.h>

unsigned short TempByte;
signed int Input1;
signed int Input2;
unsigned int Output1;
unsigned int Output2;
unsigned short CycleCounter;                                                      // Helper for 10mS Interrupt
char txt[7];

const unsigned int FailureFallbackVoltage = 0;                                  // Fallback voltage on sensor failure

void InitTimer1(){                                                              // Init Timer 1 for 10mS intervall
     T1CON      = 1;                                                            // Select timer configuration
     TMR1IF_bit = 0;                                                            // Reset timer 1 overrun bit
     TMR1H      = 0x63;
     TMR1L      = 0xC0;
     TMR1IE_bit = 1;                                                            // Enable timer 1
     PEIE_bit   = 1;                                                            // Peripheral interrupts enabled (fired by PIE2/TMR1 Interrupt)
     GIE_bit    = 1;                                                            // General: Interrupts enabled
}

void InitPWM2(){                                                                // Output 1
     PWM2CLKCON  = 0;                                                           // Config PWM 2
     PWM2LDM_bit = 0;                                                           // Config PWM 2
     PWM2CON     = 0;                                                           // Config PWM 2
     PWM2DCH     = 0;                                                           // Config PWM 2
     PWM2DCL     = 0;                                                           // Config PWM 2
     PWM2PHH     = 0;                                                           // Config PWM 2
     PWM2PHL     = 0;                                                           // Config PWM 2
     PWM2OFH     = 0;                                                           // Config PWM 2
     PWM2OFL     = 0;                                                           // Config PWM 2
     PWM2PRH     = 0x03;                                                        // Config PWM 2 Frequency = 16000 Hz
     PWM2PRL     = 0xE8;                                                        // Config PWM 2 Frequency = 16000 Hz
}

void InitPWM3(){                                                                // Output 2
     PWM3CLKCON  = 0;                                                           // Config PWM 3
     PWM3LDM_bit = 0;                                                           // Config PWM 3
     PWM3CON     = 0;                                                           // Config PWM 3
     PWM3DCH     = 0;                                                           // Config PWM 3
     PWM3DCL     = 0;                                                           // Config PWM 3
     PWM3PHH     = 0;                                                           // Config PWM 3
     PWM3PHL     = 0;                                                           // Config PWM 3
     PWM3OFH     = 0;                                                           // Config PWM 3
     PWM3OFL     = 0;                                                           // Config PWM 3
     PWM3PRH     = 0x03;;                                                       // Config PWM 3 Frequency = 16000 Hz
     PWM3PRL     = 0xE8;                                                        // Config PWM 3 Frequency = 16000 Hz
}

void InitHardware(){
     ANSELA     = 0b00000000;                                                   // All pins digital.
     PORTA      = 0b00000000;                                                   // Reset all Port Bits
     TRISA      = 0b11001010;                                                   // Select all pins as input/output
     // TRISA.0     = 0                                                         // RA0 = Output = PWM2 + TX
     // TRISA.1     = 1                                                         // RA1 = Input = RX
     // TRISA.2     = 0                                                         // RA2 = Output = PWM3
     // TRISA.3     = 1                                                         // RA3 = Input = 7.5V Adjust Jumper
     // TRISA.4     = 0                                                         // RA4 = Output = 1 Wire 2
     // TRISA.5     = 0                                                         // RA5 = Output = 1 Wire 1
     // TRISA.6     = 0                                                         // Not implemented
     // TRISA.7     = 0                                                         // Not implemented
     OSCCON =  0b01111011;                                                      // Internal Oscillator 16Mhz
     InitTimer1();                                                              // Init timer 1
     // UART1_Init(9600);
     InitPWM2();                                                                  // Init PWM2
     InitPWM3();                                                                  // Init PWM3
}

void StartReadInput1(){
     Ow_Reset(&PortA, 4);                                                       // Reset Onewire
     Ow_Write(&PortA, 4, 0xCC);                                                 // Send SKIP_ROM
     Ow_Write(&PortA, 4, 0x44);                                                 // Send CONVERT_T
}

void StartReadInput2(){
     Ow_Reset(&PortA, 5);                                                       // Reset Onewire
     Ow_Write(&PortA, 5, 0xCC);                                                 // Send SKIP_ROM
     Ow_Write(&PortA, 5, 0x44);                                                 // Send CONVERT_T
}

void GetInput1(){
     if (Ow_Reset(&PortA, 4) == 0){                                             // Reset Onewire*/
        Ow_Write(&PortA, 4, 0xCC);                                              // Send SKIP_ROM
        Ow_Write(&PortA, 4, 0xBE);                                              // Send READ_SCRATCHPAD
        Lo(Input1) = Ow_Read(&PORTA, 4);                                        // DS18B20 - Byte 0 - TEMPERATURE LSB
        Hi(Input1) = Ow_Read(&PORTA, 4);                                        // DS18B20 - Byte 1 - TEMPERATURE MSB
        Input1 = (Input1 * 10) / 16 ;
        TempByte = Ow_Read(&PortA, 4);                                          // DS18B20 - Byte 2 - TH REGISTER OR USER BYTE 1
        TempByte = Ow_Read(&PortA, 4);                                          // DS18B20 - Byte 3 - TL REGISTER OR USER BYTE 2
        TempByte = Ow_Read(&PortA, 4);                                          // DS18B20 - Byte 4 - CONFIGURATION REGISTER
        // TempByte = Ow_Read(&PortA, 4)                                        // DS18B20 - Byte 5 - RESERVED (FFh)
        if (TempByte == 0){
           Input1 = 9999;
           }
        }
    else{
         Input1 = 9999;
         }
}

void GetInput2(){
     if (Ow_Reset(&PortA, 5) == 0){                                             // Reset Onewire*/
        Ow_Write(&PortA, 5, 0xCC);                                              // Send SKIP_ROM
        Ow_Write(&PortA, 5, 0xBE);                                              // Send READ_SCRATCHPAD
        Lo(Input2) = Ow_Read(&PORTA, 5);                                        // DS18B20 - Byte 0 - TEMPERATURE LSB
        Hi(Input2) = Ow_Read(&PORTA, 5);                                        // DS18B20 - Byte 1 - TEMPERATURE MSB
        Input2 = (Input2 * 10) / 16 ;
        TempByte = Ow_Read(&PortA, 5);                                          // DS18B20 - Byte 2 - TH REGISTER OR USER BYTE 1
        TempByte = Ow_Read(&PortA, 5);                                          // DS18B20 - Byte 3 - TL REGISTER OR USER BYTE 2
        TempByte = Ow_Read(&PortA, 5);                                          // DS18B20 - Byte 4 - CONFIGURATION REGISTER
        // TempByte = Ow_Read(&PortA, 5)                                        // DS18B20 - Byte 5 - RESERVED (FFh)
        if (TempByte == 0){
            Input2 = 9999;
           }
        }
    else{
        Input2 = 9999;
        }
}
       
signed int ScaleOut(signed int InputValue){
       if(InputValue == 9999){
          return(FailureFallbackVoltage);                                       // Failsafe Output
          }

       if (InputValue < -200){ //-20C
          InputValue = -200;
          }

       if (InputValue > +800){ //80C
          InputValue = 800;
          }

       return( InputValue + 200);
}

void Set_PWM2(){                                                                // Output 1
     PWM2DCH     = Hi(Output1);                                                 // Set the new duty cycle
     PWM2DCL     = Lo(Output1);                                                 // Set the new duty cycle
     PWM2LD_bit  = 1;                                                           // Load PWM2 with new duty cycle
     PWM2OE_bit  = 1;                                                           // Enable PWM2 output
     PWM2EN_bit  = 1;                                                           // Enable PWM2
}

void Set_PWM3(){                                                                // Output 2
     PWM3DCH     = Hi(Output2);                                                 // Set the new duty cycle
     PWM3DCL     = Lo(Output2);                                                 // Set the new duty cycle
     PWM3LD_bit  = 1;                                                           // Load PWM3 with new duty cycle
     PWM3OE_bit  = 1;                                                           // Enable PWM3 output
     PWM3EN_bit  = 1;                                                           // Enable PWM3
}

void interrupt(){
  if (TMR1IF_bit){                                                              // Timer1 Interrupt - 10ms
       TMR1IF_bit = 0;
       TMR1H      = 0x63;
       TMR1L      = 0xC0;
       CycleCounter ++;
    }
}

void main() {
     InitHardware();                                                            // Init the PIC hardware
     CycleCounter = 0;
     while (1) {                                                                // Endless loop
           if (CycleCounter == 10){                                             // Start Read in
              StartReadInput1();
              StartReadInput2();
              }

        if (CycleCounter == 100){                                               // Give out
            GetInput1();
            Output1 = ScaleOut(Input1);

            GetInput2();
            Output2 = ScaleOut(Input2);

            if (PortA.B3 == 1){                                                 // 7.5V Adjust Jumper Selected
               Output1 = 750;
               Output2 = 750;
               }

            Set_PWM2();                                                         // Output 1
            Set_PWM3();                                                         // Output 2

            CycleCounter = 0;
            }

         asm CLRWDT;
     }

}